#include <unistd.h>
#include <stdio.h>
#include <math.h>

int BC(int N, double *T[]) {
//----------------------------------------------------------------------------------
// Set the boundary conditions for a discretized plate T with a 
// (N+2) x (N+2) mesh of nodes. The extremes at left/right/top/bottom of the plate
// are indexed by 0 and N+1.
//
// The boundaries are set to some nontrivial values, but are otherwise somewhat
// arbitrary. Changing them won't cause a major amount of difference in the 
// computational rates, which is what this batch of code is intended for.
//
// My recommendation: if you change the boundary conditions, rename this set so that
// it is easy to go back to the original ones.
//
//----------------
// Randall Bramley
// Department of Computer Science
// Indiana University, Bloomington
//----------------
// Last Modified: Wed 24 Jan 2018, 12:52 PM
//----------------------------------------------------------------------------------

    int i;              // loop index
    const double pi = 3.141592653589793238462643;  
    double halfN = (double)(N+1)/2.0;
    double factor = 2.0*pi/halfN;  // = (4pi/(N+1))
    double x, y;

    for (i = 0; i <= N+1; ++i) {
        // Bottom of plate: two full sine curves on [-2pi, 2pi].
        // The formula to convert from i-coordinates to x coordinates is 
        //      x = (i - (N+1)/2)*(4pi/(N+1))
        x = ((double) i - halfN)*factor;
        T[i][0] = sin(x);

        // Top of the plate: put cosine curves. Why not?
        T[i][N+1]  = cos(x);

        // Right-hand side of the plate: set to constant 1/2
        T[N+1][i] = 0.5;

        // Left-hand side of the plate: linear from 0 to 1
        y = (double)i / (double)(N+1);
        T[0][i] = y;
    }
    return 0;
}
