module kindlytypes
!--------------------------------------------------------------------------------
! Set up the kinds of integers, reals, etc., used by a project. This now uses
! the iso_fortran_env module for 4  and 8 byte real numbers, but still uses
! selected_real_kind() for 16-byters. That's because real128 may not be defined
! in all Fortran implementations of iso_fortran_env. 
!
! Beware: the actual integer value returned by selected_real_kind() is dependent
! on the compiler. So do NOT assume that real8 = 8. On SGI platforms, the returned
! integer value of real8 is 2. Also, the convention of declaring double precision 
! numbers via real*4 or real*8 is NOT a Fortran standard. Most compilers will let 
! you use those, but do not rely on it. 
!
! The trick used to get a working precision wp of maximal precision is from 
! James Van Buskirk, so give him credit for that, not me.
!
!-----------------
! Randall Bramley
! Department of Computer Science
! Indiana University, Bloomington
!----------------------
! Started: Before King Arthur started grabbing cutlery from rocks
! Modified: Sun 10 Jun 2012, 05:41 PM to add in quad prec
! Modified: Thu 12 Jan 2017, 05:11 PM to use iso_fortran_env
! Last Modified: Tue 07 Feb 2017, 02:16 PM
!--------------------------------------------------------------------------------

    use iso_fortran_env, only: real4 => real32, real8 => real64
    implicit none
    public 
    integer, parameter :: real16   = selected_real_kind(30,1000)
    !--------------------------------------------------------------------------
    ! selected_real_kind() gives a negative integer if the specs cannot be
    ! satisfied:
    !   -1 if precision is ok but exponent range is not achievable
    !   -2 if precision is not achievable but range is ok
    !   -3 if neither precision and range are not achievable 
    !   -4 a type exists for precision and a different type exists for the
    !      exponent range
    ! The sign() function returns +1 if real16 is nonnegative, -1 otherwise.
    ! So the trick here is that one of the two terms must be 0, depending only
    ! on the sign of real16 but not its particular value. 
    !--------------------------------------------------------------------------
    integer, parameter :: wp       = (1+sign(1, real16))/2*real16 + &
                                     (1-sign(1, real16))/2*real8
    
    integer, parameter :: integer4 = selected_int_kind(9)
    integer, parameter :: integer8 = selected_int_kind(18)

    
end module kindlytypes
